package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.RoundRectangle2D;

public class BasicPart extends Part {
    private static final int types[] = {Arc2D.OPEN, Arc2D.CHORD, Arc2D.PIE};
	private static int caps[] = {BasicStroke.CAP_BUTT, BasicStroke.CAP_ROUND, BasicStroke.CAP_SQUARE};
	private static int joins[] = {BasicStroke.JOIN_MITER, BasicStroke.JOIN_ROUND, BasicStroke.JOIN_BEVEL};
	
	private static final class BasicRender extends Renderer {
		private void drawArcs(Graphics2D g2, int w, int h) {
			double wa = w / 3.2;
			double ha = h / 3.2;
			
			g2.translate(wa * 0.1, ha * 0.1);
			Composite old = g2.getComposite();
			Composite comp = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f);
			g2.setStroke(new BasicStroke(8.0f));
			for (int i = 0; i < types.length; i++) {
				Arc2D arc = new Arc2D.Float(types[i]);
				arc.setFrame(i * wa, i * ha, wa , ha);
				arc.setAngleStart(45);
				arc.setAngleExtent(270);
				g2.setColor(Color.red);
				g2.fill(arc);
				g2.setColor(Color.blue);
				g2.setComposite(comp);
				g2.draw(arc);
				g2.setComposite(old);
				g2.setColor(Color.black);
			}
		}
		
		private void fillWindingRule(Graphics2D g2, int w, int h) {
			GeneralPath path = new GeneralPath();
			
			h = h / 2;
			int rx = w / 3;
			int ry = h / 2;
			path.append(new Ellipse2D.Float(10, 10, w - rx, h - ry), false);
			path.append(new Ellipse2D.Float(10 + rx / 2, 10 + ry / 2, w - rx, h - ry), false);
			
			g2.setColor(Color.gray);
			g2.fill(path);
			g2.setColor(Color.black);
			g2.draw(path);
			
			g2.translate(0, h);
			path.setWindingRule(GeneralPath.WIND_EVEN_ODD);
			g2.setColor(Color.gray);
			g2.fill(path);
			g2.setColor(Color.black);
			g2.draw(path);
		}
		
		private void drawDashs(Graphics2D g2, int w, int h) {
			int space = 3;
			BasicStroke dotted = new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{0, 6, 0, 6}, 0);
			g2.setStroke(dotted);
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			g2.setStroke(new BasicStroke(0, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1, 6, 1, 6}, 0));
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			g2.setStroke(new BasicStroke(0.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1, 6, 1, 6}, 0));
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			g2.setStroke(new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1, 6, 1, 6}, 0));
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			g2.setStroke(new BasicStroke(2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1, 6, 1, 6}, 0));
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			g2.setStroke(new BasicStroke(3, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND, 0, new float[]{1, 6, 1, 6}, 0));
			g2.drawRect(space, space, w - space * 2, h - space * 2);
			space += 6;
			int w1 = w - space * 2;
			int h1 = h - space * 2;
			g2.setStroke(new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10, new float[]{1.1f}, 0));
			g2.draw(new RoundRectangle2D.Float(space, space, w1, h1, 12, 12));
			space += 10;
			g2.setStroke(new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10, new float[]{2.1f}, 0));
			g2.draw(new Arc2D.Float(space, space, w - space * 2, h - space * 2, 45, 270, Arc2D.PIE));
			g2.setStroke(new BasicStroke(3, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10, new float[]{3.1f}, 0));
			g2.drawOval(space + w1 / 4, space + h1 / 4 - h1 / 10, w1 / 5, h1 / 5);
		}
		
		private void drawStroke(Graphics2D g2, int w, int h) {
			int lineWidth = 20;
			int space = lineWidth * 2;
			
			int y = 0;
			for (int i = 0; i < caps.length; i++) {
				g2.setStroke(new BasicStroke(lineWidth, caps[i], BasicStroke.JOIN_MITER));
				y = lineWidth * (1 + 2 * i);
				g2.drawLine(space, y, w - space, y);
			}
			y += space;
			GeneralPath path = new GeneralPath();
			path.moveTo(w - space, y);
			path.lineTo(space, y);
			path.lineTo(space + w / 2, y + h / 4);
			for (int i = 0; i < joins.length; i++) {
				g2.setStroke(new BasicStroke(lineWidth, BasicStroke.CAP_BUTT, joins[i]));
				g2.draw(path);
				g2.translate(0, space + lineWidth);
			}
		}
		
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			int w1 = w / 2;
			int h1 = h / 2;
			g2.drawLine(0, h1, w, h1);
			g2.drawLine(w1, 0, w1, h);
			
			Graphics2D g = (Graphics2D) g2.create(0, 0, w1, h1);
			drawArcs(g, w1, h1);
			g.dispose();
			g = (Graphics2D) g2.create(w1, 0, w1, h1);
			fillWindingRule(g, w1, h1);
			g.dispose();
			g = (Graphics2D) g2.create(0, h1, w1, h1);
			drawStroke(g, w1, h1);
			g.dispose();
			g = (Graphics2D) g2.create(w1, h1, w1, h1);
			drawDashs(g, w1, h1);
			g.dispose();
			/*
			g2.setStroke(new BasicStroke(20));
			g2.setClip(new Ellipse2D.Float(10, 10, 100, 100));
			//g2.setClip(new QuadCurve2D.Float(10, 100, 50, 100, 100, 100));
			
			g2.setPaint(Color.red);
			g2.fillRect(0, 0, 200, 200);
			*/
		}
	}
	
	protected Renderer createRenderer() {
		return new BasicRender();
	}
}
