package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import com.spket.g2d.G2DRenderingHints;
import com.spket.g2d.NativeImage;

public class ClippingPart extends Part {
	private class Clipping extends Renderer {
		public void render(Graphics2D g, boolean awt, int w, int h) {
			if (showPrblem) {
				drawProblem(g, awt, w, h);
				return;
			}
			if (applyClip && clip != null) {
				if (bgType == 1) {
					g.setPaint(Color.blue.brighter());
					g.fillRect(0, 0, w, h);
				}
				
				Rectangle2D rect = clip.getBounds2D();
				AffineTransform tx = g.getTransform();
				g.translate(10, 10);
				g.scale((w - 20) / rect.getWidth(), (h - 20) / rect.getHeight());
				g.translate(-rect.getX(), -rect.getY());
				g.setClip(clip);
				g.setTransform(tx);
				
				if (!awt)
					g.setRenderingHint(G2DRenderingHints.KEY_CLIP_ANTIALIASING, antialiasing ? G2DRenderingHints.VALUE_CLIP_ANTIALIASING_ON : G2DRenderingHints.VALUE_CLIP_ANTIALIASING_OFF);
			}
			
			switch (bgType) {
				case 1: // Texture
				{
					BufferedImage texture = getTexture();
					if (texture == null) {
						g.setPaint(Color.black);
						drawString(g, "Unable to load texture", w,  h);
						return;
					}
					g.setPaint(Color.white);
					g.fillRect(0, 0, w, h);
					g.setPaint(new TexturePaint(texture, new Rectangle2D.Float(0, 0, texture.getWidth(), texture.getHeight())));
					g.fillRect(0, 0, w, h);
				}
					break;
				case 2: // Image
					if (background == null)
						return;
					
					g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
					g.drawImage(background, 0, 0, w, h, null);
					break;
				default: // Gradient
					g.setPaint(new GradientPaint(0, 0, new Color(255, 0, 0, 200), w, h, new Color(0, 0, 255, 120)));
					g.fillRect(0, 0, w, h);
					break;
			}
		}
		
		protected void drawProblem(Graphics2D g, boolean awt, int w, int h) {
			if (awt) {
				BufferedImage image = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
				Graphics2D g2 = image.createGraphics();
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
				draw(g2, w, h);
				
				g2.dispose();
			
				g.drawImage(image, 0, 0, null);
			} else {
				NativeImage image = new NativeImage(w, h);
				
				Graphics2D g2 = image.createGraphics();
				g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
				g2.setRenderingHint(G2DRenderingHints.KEY_CLIP_ANTIALIASING, antialiasing ? G2DRenderingHints.VALUE_CLIP_ANTIALIASING_ON : G2DRenderingHints.VALUE_CLIP_ANTIALIASING_OFF);
			
				draw(g2, w, h);
				
				g2.dispose();
			
				g.drawImage(image, 0, 0, null);
				image.dispose();
			}
		}
		
		protected void draw(Graphics2D g2, int width, int height) {
			int x = 10;
			int y = 10;
			int r = 50;
			int w1 = r * 2;
			int h1 = r * 2;
			
			Area area = new Area(new Ellipse2D.Double(x, y, w1,  h1));
			area.add(new Area(new Ellipse2D.Double(x + r, y + r, w1, h1)));
			
			g2.setPaint(Color.white);
			g2.fillRect(0, 0, width, height);
			
			g2.setPaint(Color.black);
			g2.setComposite(AlphaComposite.Clear);
			g2.fill(area);
			
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN));
			g2.setPaint(Color.blue);
			g2.fill(area);
			
			g2.translate(w1 + r, 0);
			
			g2.setClip(area);
			g2.setPaint(Color.black);
			g2.setComposite(AlphaComposite.Clear);
			g2.fillRect(0, 0, width, height);
			
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN));
			g2.setPaint(Color.blue);
			g2.fillRect(0, 0, width, height);
			
			y += 200;
			g2.translate(-(w1 + r), 0);
			Rectangle2D.Double rect = new Rectangle2D.Double(x + 0.5, y + 0.5, width - 20.5, height - y - 20.5);
			g2.setClip(rect);
			g2.setPaint(Color.white);
			g2.setComposite(AlphaComposite.Clear);
			g2.fillRect(0, 0, width, height);
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN));
			g2.setPaint(Color.blue);
			g2.fillRect(0, 0, width, height);
		}
		
	}
	
	private static final String TEXT = "OopS~";
	
	private static Shape shape;
	
	private Control client;
	
	private int bgType;
	
	private String text;
	private BufferedImage background;
	
	private Shape clip;
	private int clipType;
	
	private boolean applyClip;
	private boolean antialiasing;
	
	private boolean showPrblem;
	
	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			switch (event.type) {
				case SWT.Selection :
					onButtonClickd(event);
					break;
				case SWT.Modify:
					text = ((Text) event.widget).getText().trim();
					if (clipType == 1) {
						clip = getOutline();
						if (applyClip && !showPrblem)
							redraw();
					}
					break;
			}
		}
	};
	
	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout(2, false));
		
		client = new Label(container, SWT.NONE);
		client.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout());
		pane.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		Group group = new Group(pane, SWT.NONE);
		group.setText("Clip"); //text or area
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button button = new Button(group, SWT.CHECK);
		button.setText("clip");
		button.setData(Boolean.FALSE);
		button.setSelection(applyClip);
		button.addListener(SWT.Selection, listener);
		
		button = new Button(group, SWT.CHECK);
		button.setText("anti-aliasing");
		button.setData(Boolean.TRUE);
		button.setSelection(antialiasing);
		button.addListener(SWT.Selection, listener);
		
		button = new Button(group, SWT.RADIO);
		button.setText("Shape");
		button.setData(new Integer(10));
		button.setSelection(true);
		button.addListener(SWT.Selection, listener);
		
		button = new Button(group, SWT.RADIO);
		button.setText("Text");
		button.setData(new Integer(11));
		button.addListener(SWT.Selection, listener);
		
		Text text = new Text(group, SWT.SINGLE | SWT.BORDER);
		text.setText(TEXT);
		text.setTextLimit(12);
		text.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		text.addListener(SWT.Modify, listener);
		
		group = new Group(pane, SWT.NONE);
		group.setText("Background");
		group.setLayout(new GridLayout());
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		button = new Button(group, SWT.RADIO);
		button.setText("Default");
		button.setData(new Integer(0));
		button.setSelection(true);
		button.addListener(SWT.Selection, listener);
		
		button = new Button(group, SWT.RADIO);
		button.setText("Texture");
		button.setData(new Integer(1));
		button.addListener(SWT.Selection, listener);
		
		button = new Button(group, SWT.RADIO);
		button.setText("Image");
		button.setData(new Integer(2));
		button.addListener(SWT.Selection, listener);
		
		button = new Button(pane, SWT.CHECK);
		button.setText("Problem");
		button.setSelection(showPrblem);
		button.addListener(SWT.Selection, listener);
		
		return container;
	}

	protected Control getClient() {
		return client;
	}

	protected Renderer createRenderer() {
		if (shape == null) {
			Area area = new Area(new Ellipse2D.Float(10, 10, 50, 50));
			area.add(new Area(new RoundRectangle2D.Float(40, 40, 60, 80, 18, 20)));
			area.subtract(new Area(new Rectangle2D.Float(20, 30, 20, 10)));
			area.subtract(new Area(new Ellipse2D.Float(60, 45, 30, 40)));
			area.subtract(new Area(new Arc2D.Float(50, 70, 40, 60, 0, 330, Arc2D.PIE)));
			
			shape = area;
		}
		clip = shape;
		background = getDefaultImage();
		
		return new Clipping();
	}
	
	protected void onButtonClickd(Event event) {
		Object data = event.widget.getData();
		if (data instanceof Integer) {
			int v = ((Integer) data).intValue();
			switch (v) {
				case 0:
				case 1:
				case 2:  //background
					if (v != bgType) {
						bgType = v;
						if (!showPrblem)
							redraw();
					}
					break;
				case 10: //clip by shape
					if (clipType == 0)
						return;
					
					clipType = 0;
					clip = shape;
					if (applyClip && !showPrblem)
						redraw();
					break;
				case 11: //clip by text
					if (clipType == 1)
						return;
					
					clipType = 1;
					clip = getOutline();
					if (applyClip && !showPrblem)
						redraw();
					break;
			}
		} else if (data instanceof Boolean) {
			if (data == Boolean.TRUE) {
				antialiasing = !antialiasing;
				if (applyClip || showPrblem)
					redraw();
			} else {
				applyClip = !applyClip;
				if (!showPrblem)
					redraw();
			}
		} else {
			showPrblem = !showPrblem;
			redraw();
		}
	}
	
	protected Shape getOutline() {
		if (text == null || text.length() == 0)
			text = TEXT;
		
		Font f = new Font(null, Font.BOLD, 72);
		return f.createGlyphVector(new FontRenderContext(null, true, true), text).getOutline();
	}
}
