package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import org.apache.batik.ext.awt.LinearGradientPaint;
import org.apache.batik.ext.awt.RadialGradientPaint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;

import com.spket.g2d.G2DComposite;
import com.spket.g2d.Gradient;

public class CompositingPart extends Part {
	static final byte[] gradients = {
		Gradient.RADIAL, Gradient.LINEAR, Gradient.DIAMOND, Gradient.CONIC, Gradient.XY, Gradient.SQRT_XY
	};
	
	static final String[] g_labels = {
		"radial", "linear", "diamond", "conic", "xy", "sqrt_xy"
	};
	
	static final int[] composites = {
		G2DComposite.CLEAR, G2DComposite.SRC, G2DComposite.DST, G2DComposite.SRC_OVER,
		G2DComposite.DST_OVER, G2DComposite.SRC_IN, G2DComposite.DST_IN, G2DComposite.SRC_OUT,
		G2DComposite.DST_OUT, G2DComposite.SRC_ATOP, G2DComposite.DST_ATOP, G2DComposite.XOR,
		G2DComposite.PLUS, G2DComposite.MINUS, G2DComposite.MULTIPLY, G2DComposite.SCREEN,
		G2DComposite.OVERLAY, G2DComposite.DARKEN, G2DComposite.LIGHTEN, G2DComposite.COLOR_DODGE,
		G2DComposite.COLOR_BURN, G2DComposite.HARD_LIGHT, G2DComposite.SOFT_LIGHT, G2DComposite.DIFFERENCE,
		G2DComposite.EXCLUSION, G2DComposite.CONTRAST
	};
	
	static final String[] labels = {
		"clear", "src", "dst", "src_over", "dst_over", "src_in", "dst_in",
		"src_out", "dst_out", "src_atop", "dst_atop", "xor", "plus", "minus",
		"multiply", "screen", "overlay", "darken", "lighten", "color_dodge",
		"color_burn", "hard_light", "soft_light", "difference", "exclusion", "contrast"
	};
	
	private static final int G_SIZE = 255;
	
	private static final int[] ratios = { 0, 85, 170, G_SIZE };
	private static final float[] fractions = { 0, 85.0f / G_SIZE, 170.0f / G_SIZE, 1.0f };
	private static final Color[] colors = { Color.black, Color.blue, Color.green, new Color(255, 0, 0, 0) };
	
	private class Compositing extends Renderer {
		protected void render(Graphics2D g2, boolean awt, double x, double y, double w, double h) {
			double x1 = x + w / 2;
			double y1 = y + h / 2;
			double y2 = y1 + h / 2;
			if (Gradient.XY == gradient || Gradient.SQRT_XY == gradient) {
				y2 = y1 + Math.sqrt(w * w + h * h) / 4;
			} else if (Gradient.LINEAR == gradient) {
				x1 = x;
				y1 = y;
			}
			if (awt) {
				Point2D.Double p1 = new Point2D.Double(x1, y1);
				Point2D.Double p2 = new Point2D.Double(x1, y2);
				if (gradient == Gradient.LINEAR) {
					g2.setPaint(new LinearGradientPaint(p1, p2, fractions, colors, LinearGradientPaint.NO_CYCLE, LinearGradientPaint.SRGB));
				} else {
					g2.setPaint(new RadialGradientPaint(p1, (float) (y2 - y1), p1, fractions, colors, LinearGradientPaint.NO_CYCLE, LinearGradientPaint.SRGB));
				}
			} else {
				g2.setPaint(new Gradient(x1, y1, x1, y2, ratios, colors, gradient, Gradient.NO_CYCLE, null));
			}
	        g2.fill(new Ellipse2D.Double(x, y, w, h));
		}
		
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			if (w < 20 || h < 20)
				return;
			
			if (!check(awt)) {
				drawString(g2, "Not Support", w, h);
				return;
				/*
				g2.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
				g2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
				g2.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
				*/
			}
			double x = 10;
			double y = 10;
			
			g2.scale(w / 340.0, h / 340.0);
			float alpha = Math.max(0, Math.min(dstAlpha, 255)) / 255.0f;
			if (awt || !modeAGG) {
				g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
			} else {
				g2.setComposite(G2DComposite.getInstance(G2DComposite.DIFFERENCE, alpha));
			}
			render(g2, awt, x, y, 320, 320);
			alpha = Math.max(0, Math.min(srcAlpha, 255)) / 255.0f;
			g2.setComposite(getComposite(awt, composite, srcAlpha));
			render(g2, awt, x + 50, y + 130, 140, 140);
			render(g2, awt, x + 130, y + 130, 140, 140);
			render(g2, awt, x + 50, y + 50, 140, 140);
			render(g2, awt, x + 130, y + 50, 140, 140);
		}

		protected boolean check(boolean awt) {
			if (awt) {
				if ((Gradient.RADIAL == gradient || Gradient.LINEAR == gradient))
					return supportComposite(awt, composite, srcAlpha);
				
				return false;
			}
			
			return true;
		}
	}
	
	int srcAlpha = 255;
	int dstAlpha = 255;
	
	boolean modeAGG;
	byte gradient = Gradient.RADIAL;
	int composite = G2DComposite.SRC_OVER;
	
	private Spinner srcField;
	private Spinner dstField;
	
	private Control canvas;

	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			if (SWT.Selection == event.type) {
				Object data = event.widget.getData();
				
				if (data instanceof Integer) {
					int comp = ((Integer) data).intValue();
					if (comp == composite)
						return;
					
					composite = comp;
					redraw();
				} else if (data instanceof Byte) {
					byte gd = ((Byte) data).byteValue();
					if (gd == gradient)
						return;
					
					gradient = gd;
					redraw();
				} else if (data == Boolean.TRUE) {
					modeAGG = !modeAGG;
					redraw();
				}
			} else {
				if (event.widget == srcField) {
					int alpha = srcField.getSelection();
					if (alpha != srcAlpha) {
						srcAlpha = alpha;
						redraw();
					}
				} else {
					int alpha = dstField.getSelection();
					if (alpha != dstAlpha) {
						dstAlpha = alpha;
						redraw();
					}
				}
			}
		}
	};

	public Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout(2, false));
		
		canvas = new Label(container, SWT.NO_BACKGROUND);
		canvas.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout());
		pane.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		Group group = new Group(pane, SWT.NONE);
		group.setText("&Composite");
		group.setLayout(new GridLayout(2, true));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button button;
		for (int i = 0; i < composites.length; i++) {
			button = new Button(group, SWT.RADIO);
			button.setText(labels[i]);
			button.setData(new Integer(composites[i]));
			button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			if (composite == composites[i])
				button.setSelection(true);
			button.addListener(SWT.Selection, listener);
		}
		
		group = new Group(pane, SWT.NONE);
		group.setText("&Gradient");
		group.setLayout(new GridLayout(2, true));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		for (int i = 0; i < gradients.length; i++) {
			button = new Button(group, SWT.RADIO);
			button.setText(g_labels[i]);
			button.setData(new Byte(gradients[i]));
			button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			if (gradient == gradients[i])
				button.setSelection(true);
			button.addListener(SWT.Selection, listener);
		}
		
		pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout());
		pane.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		group = new Group(pane, SWT.NONE);
		group.setText("&Alpha");
		group.setLayout(new GridLayout(5, false));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		button = new Button(group, SWT.CHECK);
		button.setText("AGG Mode");
		button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		button.setData(Boolean.TRUE);
		button.setSelection(modeAGG);
		button.addListener(SWT.Selection, listener);
		
		Label label = new Label(group, SWT.NONE);
		label.setText("Src");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		srcField = new Spinner(group, SWT.BORDER);
		srcField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		srcField.setMinimum(0);
		srcField.setMaximum(255);
		srcField.setIncrement(1);
		srcField.setSelection(255);
		srcField.addListener(SWT.Modify, listener);
		
		label = new Label(group, SWT.NONE);
		label.setText("Dst");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		dstField = new Spinner(group, SWT.BORDER);
		dstField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		dstField.setMinimum(0);
		dstField.setMaximum(255);
		dstField.setIncrement(1);
		dstField.setSelection(255);
		dstField.addListener(SWT.Modify, listener);
		
		return container;
	}

	protected Control getClient() {
		return canvas;
	}

	protected Renderer createRenderer() {
		return new Compositing();
	}
	/*
	public void testPerformance() {
		int width = 1024;
		int height = 768;
		double[][] values = new double[gradients.length][composites.length];
		Display display = new Display();
		RenderBuffer buf = new RenderBuffer(display, width, height);
		Graphics2D g = buf.createGraphics();
		
		long t;
		try {
			for (int i = 0; i < gradients.length; i++) {
				gradient = gradients[i];
				for (int j = 0; j < composites.length; j++) {
					composite = composites[j];
					g.setPaint(Color.white);
					g.setComposite(G2DComposite.SrcOver);
					g.fillRect(0, 0, width, height);
					t = System.nanoTime();
					render(g, width, height);
					values[i][j] = (System.nanoTime() - t) / 1000000.0;
					Thread.sleep(20);
				}
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		g.dispose();
		buf.dispose();
		display.dispose();
		for (int i = 0; i < gradients.length; i++) {
			System.out.println(g_labels[i]);
			for (int j = 0; j < composites.length; j++) {
				System.out.print('\t');
				System.out.println(labels[j] + ": " + values[i][j]);
			}
		}
		for (int j = 0; j < composites.length; j++) {
			System.out.println(labels[j]);
			for (int i = 0; i < gradients.length; i++) {
				System.out.print('\t');
				System.out.println(g_labels[i] + ": " + values[i][j]);
			}
		}
	}
	*/
}
