package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.TexturePaint;
import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.DirectColorModel;
import java.awt.image.WritableRaster;

import org.apache.batik.ext.awt.RadialGradientPaint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import com.spket.g2d.G2DComposite;
import com.spket.g2d.G2DRenderingHints;
import com.spket.g2d.Gradient;
import com.spket.g2d.NativeImage;
import com.spket.g2d.SWTImage;
import com.spket.g2d.Texture;
import com.spket.g2d.internal.G2D;

public class G2DTester {
	public static void main(String[] args) {
		/*
		BufferedImage img = new BufferedImage(10, 10, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = img.createGraphics();
		g.setPaint(Color.black);
		g.fillRect(0, 0, 10 ,10);
		g.dispose();
		test1();
		
		//test();
		System.out.println(G2D.queryPerformanceFrequency());
		System.out.println(G2D.queryPerformanceCounter());
		System.out.println(System.currentTimeMillis());
		
		testPerformance();
		
		System.out.println(G2D.queryPerformanceCounter());
		/*
		*/
		G2DTester tester = new G2DTester();
		
		try {
			tester.run();
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}
	
	public static void testPerformance() {
		int width = 1024;
		int height = 768;
		byte[] gradients = CompositingPart.gradients;
		int[] composites = CompositingPart.composites;
		final int[] ratios = { 0, 255 };
		final Color[] colors = { new Color(255, 128, 128, 250), new Color(128, 255, 255, 255) };
		
		double[][] values = new double[gradients.length][composites.length];
		NativeImage buf = new NativeImage(width, height);
		Graphics2D g = buf.createGraphics();
		
		long t;
		try {
			for (int i = 0; i < gradients.length; i++) {
				byte gradient = gradients[i];
				//for (int j = 0; j < composites.length; j++) {
					int j = 9;
					int gap = 10;
					float alpha = 0.9f;
					int composite = composites[j];
					t = Timing.counter();
					for (int c = 0; c < 10; c++) {
						g.setPaint(new Gradient(gap, gap, width - gap, height - gap, ratios, colors, gradient, Gradient.NO_CYCLE, null));
						g.setComposite(G2DComposite.getInstance(composite, alpha));
						g.fillOval(gap, gap, width - gap * 2, height - gap * 2);
						gap += 5;
						alpha -= 0.07f;
					}
					values[i][j] = Timing.time(t);
					buf.clear(Color.white);
					Thread.sleep(20);
				//}
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		g.dispose();
		buf.dispose();
		for (int i = 0; i < gradients.length; i++) {
			System.out.println(CompositingPart.g_labels[i]);
			for (int j = 0; j < composites.length; j++) {
				System.out.print('\t');
				System.out.println(CompositingPart.labels[j] + ": " + values[i][j]);
			}
		}
		for (int j = 0; j < composites.length; j++) {
			System.out.println(CompositingPart.labels[j]);
			for (int i = 0; i < gradients.length; i++) {
				System.out.print('\t');
				System.out.println(CompositingPart.g_labels[i] + ": " + values[i][j]);
			}
		}
	}

	
	protected static void test() {
		Display display = new Display();
		
		int w = 1024 / 10;
		int h = 768 / 10;
		NativeImage buf = new NativeImage(w, h, true);
		Graphics2D g = buf.createGraphics();
		g.dispose();
		buf.dispose();
		
		long t = Timing.counter();
		for (int i = 0; i < 24; i++) {
			buf = new NativeImage(w, h, true);
			buf.dispose();
			//buf.clear(0);
		}
		double time = Timing.time(t);
		System.out.println("Time: " + time);
		
		//buf.dispose();
		display.dispose();
	}
	
	protected static void test1() {
		Ellipse2D e = new Ellipse2D.Double(0, 0, 100, 80);
		Area a = new Area(e);
		System.out.println(a.isPolygonal() + " - " + a.isRectangular() + " - " + a.isSingular());
		Rectangle2D rect = new Rectangle2D.Double(200, 100, 200, 360);
		a = new Area(rect);
		System.out.println(a.isPolygonal() + " - " + a.isRectangular() + " - " + a.isSingular());
		GeneralPath path = new GeneralPath();
		path.moveTo(0, 0);
		path.lineTo(0, 10);
		path.lineTo(10, 10);
		path.closePath();
		a = new Area(path);
		System.out.println(a.isPolygonal() + " - " + a.isRectangular() + " - " + a.isSingular());
		path.reset();
		path.moveTo(0, -5);
		path.lineTo(0, 10);
		path.lineTo(10, 10);
		path.lineTo(5, 10);
		path.lineTo(10, 10);
		path.lineTo(10, 0);
		path.lineTo(-2, 0);
		//path.lineTo(-2, 2);
		//path.closePath();
		a = new Area(path);
		System.out.println(a.isPolygonal() + " - " + a.isRectangular() + " - " + a.isSingular());
		System.out.println(a.getBounds2D());
		long t = Timing.counter();
		for (int i = 0; i < 10000; i++) {
			path.reset();
			path.append(e, false);
			path.append(rect, false);
			path.getBounds2D();
		}
		
		System.out.println("Time: " + Timing.time(t));
	}
	
	private NativeImage g2dImage;
	private BufferedImage awtImage;
	
	private SashForm form;
	private Control g2dCanvas;
	private Control awtCanvas;
	
	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			switch (event.type) {
				case SWT.Paint:
					onPaint(event, event.widget != g2dCanvas);
					break;
				case SWT.MouseDoubleClick:
					if (g2dCanvas.isVisible() && awtCanvas.isVisible()) {
						if (form.getMaximizedControl() != null)
							form.setMaximizedControl(null);
						else
							form.setMaximizedControl((Control) event.widget);
					}
					break;
			}
		}
	};
	
	protected void run() {
		Display display = new Display();
		//UIUtils.activeTheming(display);
		Shell shell = new Shell(display);
		shell.setText("Grahpics");
		//layout.marginWidth = layout.marginHeight = 0; 
		shell.setLayout(new GridLayout(4, false));
		new Label(shell, SWT.NONE).setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Button button = new Button(shell, SWT.PUSH);
		button.setText("Test");
		button.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				/*
				byte v = 8;
				int count = 10240 * 768;
				ByteBuffer buf = ByteBuffer.allocateDirect(count);
				double time = Timing.nanoTime();
				for (int i = 0; i < count; i++) {
					buf.put(v);
				}
				time = (Timing.nanoTime() - time) / 1000000;
				System.out.println(time);
				time = Timing.nanoTime();
				for (int i = 0; i < count; i++) {
					buf.put(i, v);
				}
				time = (Timing.nanoTime() - time) / 1000000;
				System.out.println(time);
				buf = null;
				byte[] bs = new byte[count];
				time = Timing.nanoTime();
				for (int i = 0; i < count; i++)
					bs[i] = (byte) ((v + i) & 0xFF);
				
				time = (Timing.nanoTime() - time) / 1000000;
				System.out.println(time);
				*/
			}
		});
		//new SymbolPicker(shell, SWT.FLAT);
		//new ColorPicker(shell, SWT.FLAT);
		///*
		form = new SashForm(shell, SWT.HORIZONTAL);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = 4;
		form.setLayoutData(gd);
		
		g2dCanvas = new Canvas(form, SWT.NO_BACKGROUND);
		g2dCanvas.addListener(SWT.Paint, listener);
		g2dCanvas.addListener(SWT.MouseDoubleClick, listener);
		awtCanvas = new Canvas(form, SWT.NO_BACKGROUND);
		awtCanvas.addListener(SWT.Paint, listener);
		awtCanvas.addListener(SWT.MouseDoubleClick, listener);
		//*/
		try {
			shell.open();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
		} finally {
			if (shell != null)
				shell.dispose();
		}
		display.dispose();
	}
	
	protected void onPaint(Event event, boolean awt) {
		int x = event.x;
		int y = event.y;
		int width = event.width;
		int height = event.height;
		if (width == 0 || height == 0)
			return;
		
		GC gc = event.gc;
		Graphics2D g2 = null;
		if (awt) {
			if (awtImage == null || awtImage.getWidth() < width || awtImage.getHeight() < height) {
				awtImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
			}
			g2 = awtImage.createGraphics();
			g2.setPaint(Color.black);
			g2.setBackground(Color.white);
			g2.clearRect(0, 0, width, height);
		} else {
			if (g2dImage == null) {
				g2dImage = new NativeImage(width, height);
			} else if (g2dImage.getWidth() < width || g2dImage.getHeight() < height) {
				g2dImage.dispose();
				g2dImage = new NativeImage(width, height);
			}
			g2 = g2dImage.createGraphics();
			g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_NORMALIZE);
		}
		Rectangle rect = ((Composite) event.widget).getClientArea();
		if (x != 0 || y != 0)
			g2.translate(-x, -y);
		g2.setClip(x, y, width, height);
		g2.clearRect(x, y, width, height);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		//double time = Timing.nanoTime();
		try {
			render(g2, !awt, rect.width, rect.height);
		} finally {
			g2.dispose();
		}
		//time = (Timing.nanoTime() - time) / 1000000.0;
		if (awt) {
			G2D.drawImage(gc, awtImage, x, y);
		} else {
			G2D.drawImage(gc, g2dImage, x, y);
		}
	}
	
	protected void render(Graphics2D g, boolean agg, int w, int h) {
		float cx = w / 2;
		float cy = h / 2;
		float r = Math.min(cx, cy) - 10;
		
		int[] ratios = {0, 220, 222, 234, 235, 236, 255};
		Color[] colors = new Color[ratios.length];
		colors[0] = Color.white;
		colors[1] = Color.white;
		colors[2] = Color.darkGray;
		colors[6] = colors[2].brighter();
		colors[5] = colors[3] = colors[0].brighter();
		colors[4] = Color.white; //colors[3].brighter();
		
		if (agg) {
			g.setPaint(new Gradient(cx, cy, r, ratios, colors, Gradient.REFLECT));
		} else {
			float[] fs = new float[ratios.length];
			for (int i = 0; i < ratios.length; i++)
				fs[i] = ((float) ratios[i]) / 255;
			
			g.setPaint(new RadialGradientPaint(cx, cy, r, fs, colors));
		}
		
		g.fill(new Ellipse2D.Double(cx - r, cy - r, r * 2, r * 2));
	}
	
	protected void testRaster(Graphics2D g) {
		ColorModel cm = new DirectColorModel(ColorSpace.getInstance(ColorSpace.CS_sRGB), 32, 0x00FF0000, 0x0000FF00, 0x000000FF, 0xFF000000, true, DataBuffer.TYPE_INT);
		WritableRaster w = cm.createCompatibleWritableRaster(200, 200);
		WritableRaster wr = w.createWritableTranslatedChild(100, 100);
		BufferedImage bi = new BufferedImage(cm, wr.createWritableTranslatedChild(0, 0), cm.isAlphaPremultiplied(), null);
		Graphics2D g2 = bi.createGraphics();
		g2.setPaint(Color.red);
		g2.fillRect(0, 0, 100, 100);
		g2.dispose();
		bi = new BufferedImage(cm, w, cm.isAlphaPremultiplied(), null);
		g.drawImage(bi, null, 0, 0);
	}
	
	protected void testGradient(Graphics2D g, boolean agg, int w, int h) {
		int cx = w / 2;
		int cy = h / 2;
		int r = 200;
		
		int fx = cx - r / 2;
		int fy = cy - r / 2;
		//g.translate(-cx / 2, -cy / 2);
		AffineTransform tx = new AffineTransform(); //AffineTransform.getScaleInstance(2, 1.5); //
		if (agg) {
			g.setPaint(new Gradient(cx, cy, r, fx, fy, GradientPart.RATIONS, GradientPart.COLORS, Gradient.REFLECT, tx));
		} else {
			Point2D c = new Point2D.Float(cx, cy);
			Point2D f = new Point2D.Float(fx, fy);
			
			g.setPaint(new RadialGradientPaint(c, r, f, GradientPart.FRACTIONS, GradientPart.COLORS, RadialGradientPaint.REFLECT, RadialGradientPaint.SRGB, tx));
		}
		g.fillRect(0, 0, w, h);
	}
	
	protected void testImage(Graphics2D g, boolean agg) {
		if (agg) {
			SWTImage img = new SWTImage("F:\\tmp\\samples\\images\\Winter.jpg");
			
			g.drawImage(img, 0, 0, null);
		} else {
			BufferedImage img = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
			
			Graphics2D g2 = img.createGraphics();
			//g2.rotate(45, 50, 50);
			g2.setPaint(Color.black);
			g2.drawRect(50, 50, 20, 40);
			g2.dispose();
			
			g.drawImage(img, 0, 0, null);
		}
		/*
		BufferedImage img = new BufferedImage(100, 100, BufferedImage.TYPE_4BYTE_ABGR_PRE);
		
		Graphics2D gm = img.createGraphics();
		gm.setPaint(Color.blue);
		gm.fillRect(0, 0, img.getWidth(), img.getHeight());
		gm.setPaint(new Color(255, 0, 0, 100));
		gm.fillOval(5, 5, 20, 20);
		gm.dispose();
		
		g.scale(2, 2);
		g.translate(10, 10);
		
		AffineTransform tx = new AffineTransform();
		tx.translate(10, 10);
		tx.scale(2, 2);
		tx.rotate(45);
		
		g.drawImage(img, tx, null);
		*/
	}
	
	protected void testTexturePaint(Graphics2D g, boolean agg) {
		BufferedImage img = new BufferedImage(20, 20, BufferedImage.TYPE_INT_ARGB);
		Graphics2D gd = img.createGraphics();
		gd.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		gd.setPaint(Color.black);
		gd.rotate(45, 7, 7);
		gd.fillRect(1, 1, 9, 9);
		gd.dispose();
		
		g.drawRect(10, 10, 400, 400);
		
		if (agg)
			g.setPaint(new Texture(img, new Rectangle2D.Float(0, 0, 20, 20), Texture.REFLECT, Texture.REFLECT));
		else
			g.setPaint(new TexturePaint(img, new Rectangle2D.Float(0, 0, 20, 20)));
		
		//g.scale(5, 5);
		//g.rotate(45);
		
		g.fillRect(0, 0, 400, 400);
	}
	
	protected void testTrans(Graphics2D g, boolean agg) {
		if (agg) {
			g.setRenderingHint(G2DRenderingHints.KEY_PIXEL_OFFSET, G2DRenderingHints.VALUE_OFFSET_TRANS_BOTH);
		}
		g.drawRect(10, 10, 10, 10); //fill, no curve, no trans

		//no scale, no shear
		g.translate(10, 10);
		g.drawRect(10, 10, 10, 10);
		
		g.rotate(45);
		g.drawRect(10, 10, 80, 80);
		
		g.scale(1, -1);
		g.drawRect(10, 10, 20, 20);
		
		g.shear(10, 10);
		g.drawRect(10, 10, 10, 10);
		
		g.setTransform(new AffineTransform());
		
		if (!agg) {
			BufferedImage img = new BufferedImage(100, 100, BufferedImage.TYPE_INT_ARGB);
			Graphics2D g2 = img.createGraphics();
			g2.setPaint(Color.blue);
			g2.fillRect(0, 0, 100, 100);
			g2.setPaint(Color.red);
			try {
				g2.copyArea(-10, 0, 100, 100, 30, 0);
			} finally {
				g2.dispose();
			}
			g.drawImage(img, null, 0, 0);
		}
	}
	
	protected void testBasic(Graphics2D g, boolean agg) {
		int x = 10, y = 10, width = 100, height = 80, gap = 20;
		
		g.drawLine(x, y, x + width, y);
		
		x += width + gap;
		
		g.drawRect(x, y, width, height);
		
		x += width + gap;
		g.drawRoundRect(x, y, width, height, 20, 20);
		
		x = 10 + width + gap;
		y += height + gap;
		height = 0;
		
		g.drawRect(x, y, width, height);
		
		x += width + gap;
		g.drawRoundRect(x, y, width, height, 20, 20);
		//g.scale(10, 30);
		
		x = 10;
		y += height + gap;
		height = 80;
		g.drawOval(x, y, width, height);
		
		x += width + gap;
		g.drawArc(x, y, width, height, 0, 135);
		
		x = 10;
		y += height + gap;
		g.fillRect(x, y, width, height);
		
		x += width + gap;
		g.fillRoundRect(x, y, width, height, 20, 20);
		
		g.draw(new Arc2D.Float(100, 100, 200, 400, 0, 90, Arc2D.OPEN));
	}
	
	protected void testLargeScale(Graphics2D g, boolean agg) {
		Graphics2D g1 = (Graphics2D) g.create();
		g1.scale(10, 30);
		g1.drawArc(2, 2, 10, 10, 0, 90);
		g1.dispose();
		
		int cx = (2 + 10 / 2) * 10;
		int cy = (2 + 10 / 2) * 30;
		g.drawLine(cx, cy, cx + 200, cy);
		g.drawLine(cx, cy, cx, cy - 200);
		
		g.drawRect(2 * 10, 2 * 30, 10 * 10, 10 * 30);
		
		g.scale(10, 30);
		g.translate(12, 0);
		
		g.draw(new Arc2D.Float(2, 2, 10, 10, 0, 90, Arc2D.OPEN));
	}
	
	protected void testGradient(Graphics2D g, boolean agg) {
		int x = 10, y = 10,  width = 100, height = 80, gap = 20;
		/*
		g.setPaint(new GradientPaint(200, 200, Color.red, 300, 300, Color.blue, true));
		
		g.rotate(Math.PI / 4, 200, 200);
		
		g.fillRect(200, 200, 220, 220);
		*/
		if (agg) {
			int index = 0;
			int tx = 0, ty = 0;
			int[] rations = { 0, 40, 150,  255 };
			AffineTransform mtx = new AffineTransform();
			Color[] colors = { Color.red, Color.yellow, Color.blue, Color.magenta };
			for (byte t = Gradient.RADIAL; t <= Gradient.SQRT_XY; t++, index++) {
				g.setPaint(new Gradient(x + width, y + height, x + width / 2, y + height / 2, rations, colors, t, Gradient.REFLECT, null));
				g.setTransform(mtx);
				g.fillRect(x, y, width * 2, height * 2);
				if ((index % 2) != 0) {
					tx = 0;
					ty += height * 2 + gap;
				} else {
					tx += width * 2 + gap;
				}
				mtx.setToIdentity();
				mtx.translate(tx, ty);
			}
		} else {
			g.setPaint(new GradientPaint(x, y, Color.red, x + width, y + height, Color.blue, true));
			g.fillOval(x, y, width, height);
		}
	}
	
	protected void testComposite(Graphics2D g, boolean agg) {
		int x = 2;
		int y = 2;
		int w = 50;
		int h = 50;
		int r = 25;
		
		Graphics2D old = g;
		BufferedImage img = null;
		if (!agg) {
			img = new BufferedImage(300, 300, BufferedImage.TYPE_4BYTE_ABGR);
			g = img.createGraphics();
		}
		g.scale(5, 5);
		g.setPaint(Color.white);
		g.fillRect(0, 0, w * 2, h * 3);
		g.setPaint(Color.black);
		Rectangle2D.Double rect = new Rectangle2D.Double(x, y, w, h);
		g.setComposite(AlphaComposite.Clear);
		g.fill(rect);
		g.setColor(Color.blue);
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN));
		g.fill(rect);
		
		y += h + 2;
		
		Ellipse2D.Double e = new Ellipse2D.Double(x, y, r * 2, r * 2);
		g.setComposite(AlphaComposite.Clear);
		g.fill(e);
		g.setColor(Color.blue);
		g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_IN, 0.5f));
		g.fill(e);
		if (!agg) {
			g.dispose();
			g = old;
			g.drawImage(img, null, 0, 0);
		}
	}
	
	protected void testStroke(Graphics2D g, boolean agg) {
		float[] dashs = { 2, 1, 3 };
		
		if (agg) {
			g.setRenderingHint(G2DRenderingHints.KEY_PIXEL_OFFSET, G2DRenderingHints.VALUE_OFFSET_TRANS_BOTH);
			g.setRenderingHint(G2DRenderingHints.KEY_FIXED_LINE_WIDTH, G2DRenderingHints.VALUE_FIXED_LINE_WIDTH_ON);
		} else {
			g.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_NORMALIZE);
		}
		
		g.setStroke(new BasicStroke(1.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, dashs, 0.0f));
		
		g.drawLine(1, 3, 100, 3);
		
		g.scale(10, 10);
		
		g.drawRect(1, 2, 20, 20);
		
		g.drawLine(1, 25, 30, 25);
	}
}
