package com.spket.test;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import com.spket.g2d.NativeImage;
import com.spket.g2d.internal.G2D;
import com.spket.widget.UIUtils;

public class G2Demo {
	public static void main(String[] args) {
		G2Demo demo = new G2Demo();
		demo.run();
	}
	
	private static final class DemoBackground extends Renderer {
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			g2.setPaint(new GradientPaint(0, 0, Color.red, w, h, Color.blue));
			g2.fillRect(0, 0, w, h);
		}

		public boolean support(boolean awt) {
			return !awt;
		}
	}
	
	private final class DemoLayout extends Layout {
		protected Point computeSize(Composite composite, int wHint, int hHint, boolean flushCache) {
			if (fPart != null) {
				Control control = fPart.getControl();
				if (Part.okToUse(control))
					return control.computeSize(wHint, hHint, flushCache);
			}
			if (Part.okToUse(form))
				return form.computeSize(wHint, hHint, flushCache);
			
			return null;
		}

		protected void layout(Composite composite, boolean flushCache) {
			Rectangle rect = composite.getClientArea();
			if (fPart != null) {
				Control control = fPart.getControl();
				if (Part.okToUse(control)) {
					control.setBounds(rect);
				}
			} else if (Part.okToUse(form)) {
				form.setBounds(rect);
			}
		}
	}
	
	private static final int[] DEFAULT_WEIGHTS = {1, 1};
	
	private static final int ACT_EXIT = Integer.MIN_VALUE;
	
	private static final int ACT_ANTI_ALIASING = 1;
	private static final int ACT_TIME = ACT_ANTI_ALIASING + 1;
	
	private static final NumberFormat formatter = new DecimalFormat("0.00");
	
	protected static Font getFont(Display display, Font font, int style, int size) {
		FontData[] fontDatas = font.getFontData();
		for (int i = 0; i < fontDatas.length; i++) {
			if (fontDatas[i] != null) {
				if (style != SWT.DEFAULT)
					fontDatas[i].setStyle(style);
				if (size != SWT.DEFAULT)
					fontDatas[i].setHeight(size);
			}
		}
		return new Font(display, fontDatas);
	}
	
	private NativeImage g2dImage;
	private BufferedImage awtImage;
	
	private IPart fPart;
	private Renderer renderer;
	private Renderer background;
	
	private Shell shell;
	
	private Font font;
	private SashForm form;
	private Control g2dCanvas;
	private Control awtCanvas;
	
	private boolean showTime;
	private boolean antialiasing;
	
	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			switch (event.type) {
				case SWT.Paint:
					onPaint(event, event.widget != g2dCanvas);
					break;
				case SWT.MouseDoubleClick:
					if (g2dCanvas.isVisible() && awtCanvas.isVisible()) {
						if (form.getMaximizedControl() != null)
							form.setMaximizedControl(null);
						else
							form.setMaximizedControl((Control) event.widget);
					}
					break;
				case SWT.Resize:
					onResize(event);
					break;
				case SWT.Selection:
					onMenuClicked(event);
					break;
			}
		}
	};
	
	public G2Demo() {
		renderer = background = new DemoBackground();
		background.setHost(this);
	}

	protected void run() {
		Display display = new Display();
		UIUtils.activeTheming(display);
		font = getFont(display, display.getSystemFont(), SWT.BOLD, SWT.DEFAULT);
		shell = new Shell(display);
		shell.setText("Grahpics 2D Demo");
		shell.setLayout(new DemoLayout());
		initMenus(shell);
		initComposite(shell);
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		
		if (g2dImage != null)
			g2dImage.dispose();
		
		font.dispose();
		display.dispose();
	}
	
	protected void initMenus(Shell shell) {
		Menu bar = new Menu(shell, SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem menu = new MenuItem(bar, SWT.CASCADE);
		menu.setText("&File");
		Menu submenu = new Menu(shell, SWT.DROP_DOWN);
		menu.setMenu(submenu);
		MenuItem item = new MenuItem(submenu, SWT.PUSH);
		item.setText("E&xit");
		item.setData(new Integer(ACT_EXIT));
		item.addListener(SWT.Selection, listener);
		MenuLoader loader = new MenuLoader();
		try {
			loader.initMenu(bar, listener);
		} catch (Exception e) {
			e.printStackTrace();
		}
		menu = new MenuItem(bar, SWT.CASCADE);
		menu.setText("&Option");
		submenu = new Menu(shell, SWT.DROP_DOWN);
		menu.setMenu(submenu);
		item = new MenuItem(submenu, SWT.CHECK);
		item.setSelection(antialiasing);
		item.setText("&Antialiasing");
		item.setData(new Integer(ACT_ANTI_ALIASING));
		item.addListener(SWT.Selection, listener);
		item = new MenuItem(submenu, SWT.CHECK);
		item.setSelection(showTime);
		item.setText("&Show Time");
		item.setData(new Integer(ACT_TIME));
		item.addListener(SWT.Selection, listener);
	}
	
	protected void initComposite(Composite parent) {
		org.eclipse.swt.graphics.Color white = parent.getDisplay().getSystemColor(SWT.COLOR_WHITE);
		
		form = new SashForm(parent, SWT.NONE);
		g2dCanvas = new Canvas(form, SWT.NO_BACKGROUND);
		g2dCanvas.setBackground(white);
		g2dCanvas.addListener(SWT.Paint, listener);
		g2dCanvas.addListener(SWT.MouseDoubleClick, listener);
		awtCanvas = new Canvas(form, SWT.NO_BACKGROUND);
		awtCanvas.setBackground(white);
		awtCanvas.addListener(SWT.Paint, listener);
		awtCanvas.addListener(SWT.MouseDoubleClick, listener);
		awtCanvas.setVisible(false);
		form.setWeights(DEFAULT_WEIGHTS);
	}
	
	protected void onMenuClicked(Event event) {
		Object data = event.widget.getData();
		if (data instanceof Integer) {
			int v = ((Integer) data).intValue();
			switch (v) {
				case ACT_EXIT:
					shell.dispose();
					break;
				case ACT_ANTI_ALIASING:
					antialiasing = !antialiasing;
					if (renderer != null)
						renderer.redraw();
					break;
				case ACT_TIME:
					showTime = !showTime;
					if (renderer != null)
						renderer.redraw();
					break;
			}
		} else if (data instanceof String) {
			IPart part = null;
			String name = (String) data;
			if (fPart != null && fPart.getClass().getName().equals(name))
				return;
			
			try {
				part = (IPart) Class.forName(name).newInstance();
			} catch (Exception e) {
				String msg = e.getLocalizedMessage();
				if (msg == null)
					msg = "Unable to create instance of: " + data;
				
				MessageBox box = new MessageBox(shell, SWT.OK | SWT.ICON_ERROR | SWT.APPLICATION_MODAL);
				box.setText(e.getClass().getName());
				box.setMessage(msg);
				box.open();
			}
			if (part != null)
				setPart(part);
		}
	}
	
	protected void onResize(Event event) {
		if (fPart == null)
			return;
		
		if (Part.okToUse(form)) {
			Rectangle rect = fPart.getClientArea();
			if (rect == null) {
				form.setBounds(0, 0, 0, 0);
			} else {
				Control control = fPart.getControl();
				Point location = shell.toControl(control.toDisplay(rect.x, rect.y));
				rect.x = location.x;
				rect.y = location.y;
				form.setBounds(rect);
			}
		}
	}
	
	protected void onPaint(Event event, boolean awt) {
		int x = event.x;
		int y = event.y;
		int width = event.width;
		int height = event.height;
		if (width == 0 || height == 0)
			return;
		
		GC gc = event.gc;
		if (renderer == null || !renderer.support(awt)) {
			gc.fillRectangle(x, y, width, height);
			return;
		}
		Graphics2D g2 = null;
		if (awt) {
			if (awtImage == null || awtImage.getWidth() < width || awtImage.getHeight() < height) {
				awtImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB_PRE);
			}
			g2 = awtImage.createGraphics();
			g2.setPaint(Color.black);
			g2.setBackground(Color.white);
			g2.clearRect(0, 0, width, height);
		} else {
			if (g2dImage == null) {
				g2dImage = new NativeImage(width, height);
			} else if (g2dImage.getWidth() < width || g2dImage.getHeight() < height) {
				g2dImage.dispose();
				g2dImage = new NativeImage(width, height);
			}
			g2 = g2dImage.createGraphics();
			g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_NORMALIZE);
		}
		Rectangle rect = ((Composite) event.widget).getClientArea();
		if (x != 0 || y != 0)
			g2.translate(-x, -y);
		g2.setClip(x, y, width, height);
		g2.clearRect(x, y, width, height);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, antialiasing ? RenderingHints.VALUE_ANTIALIAS_ON : RenderingHints.VALUE_ANTIALIAS_OFF);
		long t = Timing.counter();
		try {
			renderer.render(g2, awt, rect.width, rect.height);
		} finally {
			g2.dispose();
		}
		double time = Timing.time(t);
		if (awt) {
			G2D.drawImage(gc, awtImage, x, y);
		} else {
			G2D.drawImage(gc, g2dImage, x, y);
		}
		if (showTime) {
			if (font != null)
				gc.setFont(font);
			
			gc.drawString("Time: " + formatter.format(time), 5, 5);
		}
	}
	
	protected void setPart(IPart part) {
		if (fPart == part)
			return;
		
		if (Part.okToUse(shell))
			shell.setRedraw(false);
		
		if (renderer != null) {
			renderer.setHost(null);
			renderer = null;
		}
		if (fPart != null) {
			fPart.removeResizeListener(listener);
			/*
			Control control = fPart.getControl();
			if (UIUtils.okToUse(control)) {
				control.removeListener(SWT.Resize, listener);
				//control.removeListener(SWT.Dispose, listener);
				control.setVisible(false);
			}
			*/
			fPart.dispose();
		}
		fPart = part;
		if (part != null) {
			Control control = part.getControl();
			if (!Part.okToUse(control))
				control = part.createControl(shell);
			
			if (Part.okToUse(control)) {
				part.addResizeListener(listener);
				control.setVisible(true);
				//control.addListener(SWT.Resize, listener);
				//control.addListener(SWT.Dispose, listener);
			}
			renderer = part.getRenderer();
			form.setMaximizedControl(null);
			form.setWeights(DEFAULT_WEIGHTS);
		}
		if (renderer == null)
			renderer = background;
		
		renderer.setHost(this);
		if (Part.okToUse(shell)) {
			if (Part.okToUse(g2dCanvas))
				g2dCanvas.setVisible(renderer.support(false));
			if (Part.okToUse(awtCanvas))
				awtCanvas.setVisible(renderer.support(true));
			if (Part.okToUse(form))
				form.layout(false);
			
			shell.layout(true);
			shell.setRedraw(true);
		}
	}
	
	protected void redraw() {
		if (Part.okToUse(g2dCanvas) && g2dCanvas.isVisible())
			g2dCanvas.redraw();
		if (Part.okToUse(awtCanvas) && awtCanvas.isVisible())
			awtCanvas.redraw();
	}
}
