package com.spket.test;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import org.apache.batik.ext.awt.LinearGradientPaint;
import org.apache.batik.ext.awt.MultipleGradientPaint;
import org.apache.batik.ext.awt.RadialGradientPaint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Spinner;

import com.spket.g2d.Gradient;

public class GradientPart extends Part {
	private static final byte[] circles = {
		Gradient.NO_CYCLE, Gradient.REPEAT, Gradient.REFLECT
	};
	
	private static final byte[] gradients = {
		Gradient.RADIAL, Gradient.LINEAR, Gradient.DIAMOND, Gradient.CONIC, Gradient.XY, Gradient.SQRT_XY
	};
	
	private static final String[] c_labels = {
		"none", "repeat", "reflect"
	};
	
	private static final String[] g_labels = {
		"radial", "linear", "diamond", "conic", "xy", "sqrt_xy"
	};
	
	private static final MultipleGradientPaint.CycleMethodEnum[] CMS = {
		MultipleGradientPaint.NO_CYCLE, MultipleGradientPaint.REPEAT, MultipleGradientPaint.REFLECT
	};
	
	private static final int MIN_RADIUS = 20;
	private static final int G_SIZE = 255;
	
	static final int[] RATIONS = { 0, 50, 90, 128, 200, G_SIZE };
	static final float[] FRACTIONS = { 0, 50f / G_SIZE, 90f / G_SIZE, 0.5f, 200f / G_SIZE, 1.0f };
	static final Color[] COLORS = { Color.black, Color.blue, Color.green, Color.yellow, Color.red, Color.white };
	
	private class GradientRenderer extends Renderer {
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			float x = w / 2;
			float y = h / 2;
			float r = radius;
			
			if (awt && !((Gradient.RADIAL == gradient || Gradient.LINEAR == gradient))) {
				drawString(g2, "Not Support", w, h);
				return;
			}
			
			if (rotation != 0)
				g2.rotate(Math.toRadians(rotation), x, y);
			
			if (awt) {
				Point2D.Double p1 = new Point2D.Double(x, y);
				if (gradient == Gradient.LINEAR) {
					Point2D.Double p2 = new Point2D.Double(x + r, y);
					g2.setPaint(new LinearGradientPaint(p1, p2, fractions, colors, CMS[circle], LinearGradientPaint.SRGB));
				} else {
					g2.setPaint(new RadialGradientPaint(p1, r, p1, fractions, colors, CMS[circle], LinearGradientPaint.SRGB));
				}
			} else {
				g2.setPaint(new Gradient(x, y, x + r, y, ratios, colors, gradient, circle, null));
			}
			double d = Math.sqrt(w * w + h * h);
			g2.fill(new Ellipse2D.Double(x - d / 2, y - d / 2, d, d));
		}
	}
	
	private Control client;
	
	private int rotation = 0;
	private int radius = MIN_RADIUS;
	
	private int[] ratios = RATIONS;
	private float[] fractions = FRACTIONS;

	private Color[] colors = COLORS;
	
	private byte circle = Gradient.NO_CYCLE;
	private byte gradient = Gradient.LINEAR;
	
	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			if (SWT.Selection == event.type) {
				Object data = event.widget.getData();
				if (data instanceof Byte) {
					byte gd = ((Byte) data).byteValue();
					if (gradient != gd) {
						gradient = gd;
						redraw();
					}
				} else if (data instanceof Integer) {
					byte v = ((Integer) data).byteValue();
					if (circle != v) {
						circle = v;
						redraw();
					}
				} else {
					int r = ((Scale) event.widget).getSelection() + MIN_RADIUS;
					if (radius != r) {
						radius = r;
						redraw();
					}
				}
			} else if (SWT.Modify == event.type) {
				int v = ((Spinner) event.widget).getSelection();
				if (rotation != v) {
					rotation = v;
					redraw();
				}
			}
		}
	};
	
	protected Renderer createRenderer() {
		return new GradientRenderer();
	}

	protected Control getClient() {
		return client;
	}

	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout(2, false));
		
		client = new Label(container, SWT.NONE);
		client.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout());
		pane.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		Group group = new Group(pane, SWT.NONE);
		group.setText("&Gradient");
		group.setLayout(new GridLayout(2, true));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Button button;
		for (int i = 0; i < gradients.length; i++) {
			button = new Button(group, SWT.RADIO);
			button.setText(g_labels[i]);
			button.setData(new Byte(gradients[i]));
			button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			if (gradient == gradients[i])
				button.setSelection(true);
			button.addListener(SWT.Selection, listener);
		}
		group = new Group(pane, SWT.NONE);
		group.setText("&Circle");
		group.setLayout(new GridLayout(1, true));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		for (int i = 0; i < circles.length; i++) {
			button = new Button(group, SWT.RADIO);
			button.setText(c_labels[i]);
			button.setData(new Integer(circles[i]));
			button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
			if (circle == circles[i])
				button.setSelection(true);
			button.addListener(SWT.Selection, listener);
		}
		
		group = new Group(pane, SWT.NONE);
		group.setText("Control");
		group.setLayout(new GridLayout(2, false));
		group.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Label label = new Label(group, SWT.NONE);
		label.setText("Rotation:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Spinner sp = new Spinner(group, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setMinimum(0);
		sp.setMaximum(360);
		sp.setIncrement(10);
		sp.setSelection(0);
		sp.addListener(SWT.Modify, listener);
		/*
		label = new Label(group, SWT.NONE);
		label.setText("Scale:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		sp = new Spinner(group, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setDigits(1);
		sp.setMinimum(5);
		sp.setMaximum(40);
		sp.setIncrement(5);
		sp.setSelection(10);
		sp.setData(Boolean.TRUE);
		sp.addListener(SWT.Modify, listener);
		*/
		label = new Label(group, SWT.NONE);
		label.setText("radius:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Scale scale = new Scale(group, SWT.VERTICAL);
		scale.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		scale.setMinimum(0);
		scale.setMaximum(500);
		scale.setIncrement(20);
		scale.addListener(SWT.Selection, listener);
		
		return container;
	}
}
