package com.spket.test;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;

import com.spket.g2d.G2DComposite;
import com.spket.g2d.G2DRenderingHints;
import com.spket.g2d.internal.G2D;

public class ImageFilterPart extends Part {
	private static final String[] g_labels = {
		"Nearest Neighbor", "Bilinear", "Bicubic", "Spline16", "Spline36", "Hanning", "Hamming", "Hermite", "Kaiser", "Quadric", "Catrom", "Gaussian", "Bessel", "Mitchell", "Sinc", "Lanczos", "Blackman"
	};
	
	private class ImageFilter extends Renderer {
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			if (image == null) {
				drawString(g2, "Unable to load image...", w, h);
				return;
			}
			if (!check(awt)) {
				drawString(g2, "Not support", w, h);
				return;
			}
			if (awt) {
				g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, getFilter(awt, filter));
			} else {
				g2.setRenderingHint(G2DRenderingHints.KEY_IMAGE_FILTER, new Integer(filter));
			}
			if (sx != 1 || sy != 1)
				g2.scale(sx != 0 ? sx : ((double) w) / image.getWidth(), sy != 0 ? sy : ((double) h) / image.getHeight());
			if (rotation != 0)
				g2.rotate(Math.toRadians(rotation), image.getWidth() / 2, image.getHeight() / 2);
			
			g2.setComposite(getComposite(awt, composite, alpha));
			g2.drawImage(image, 0, 0, null);
		}
		
		private boolean check(boolean awt) {
			return supportFilter(awt, filter) && supportComposite(awt, composite, alpha);
		}
	}
	
	private Control client;
	
	private BufferedImage image;
	
	private double sx;
	private double sy;
	private int rotation;
	
	private int filter = G2D.NEAREST_NEIGHBOR;
	
	private int alpha = 255;
	private int composite = G2DComposite.SRC_OVER;

	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			Object data = event.widget.getData();
			switch (event.type) {
				case SWT.Selection:
					if (data == null) { // filters
						int f = ((Combo) event.widget).getSelectionIndex();
						if (f != filter) {
							filter = f;
							redraw();
						}
					} else if (data == Boolean.FALSE) { // Image
						openImage();
					} else { // Composite
						int c = CompositingPart.composites[((Combo) event.widget).getSelectionIndex()];
						if (c != composite) {
							composite = c;
							redraw();
						}
					}
					break;
				case SWT.Modify:
				{
					int v = ((Spinner) event.widget).getSelection();
					if (data == null) { // rotation
						if (v != rotation) {
							rotation = v;
							redraw();
						}
					} else if (data == Boolean.TRUE) { // scale x
						double s = v / 10.0;
						if (s != sx) {
							sx = s;
							redraw();
						}
					} else if (data == Boolean.FALSE) { // scale y
						double s = v / 10.0;
						if (s != sy) {
							sy = s;
							redraw();
						}
					} else { // alpha
						if (alpha != v) {
							alpha = v;
							redraw();
						}
					}
				}
					break;
			}
		}
	};
	
	protected Renderer createRenderer() {
		image = getDefaultImage();
		
		return new ImageFilter();
	}

	protected Control getClient() {
		return client;
	}

	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout(2, false));
		
		client = new Label(container, SWT.NONE);
		client.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout(2, false));
		pane.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		
		Label label = new Label(pane, SWT.NONE);
		label.setText("Image:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Button button = new Button(pane, SWT.PUSH);
		button.setText("Browser...");
		button.setData(Boolean.FALSE);
		button.addListener(SWT.Selection, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("filter:");
		
		Combo filters = new Combo(pane, SWT.READ_ONLY);
		filters.setItems(g_labels);
		filters.select(filter);
		filters.setVisibleItemCount(g_labels.length / 2);
		filters.addListener(SWT.Selection, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("&Rotation:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Spinner sp = new Spinner(pane, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setMinimum(0);
		sp.setMaximum(360);
		sp.setIncrement(10);
		sp.setSelection(0);
		sp.addListener(SWT.Modify, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("Scale(X):");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		sp = new Spinner(pane, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setDigits(1);
		sp.setMinimum(0);
		sp.setMaximum(40);
		sp.setIncrement(5);
		sp.setSelection(0);
		sp.setData(Boolean.TRUE);
		sp.addListener(SWT.Modify, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("Scale(Y):");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		sp = new Spinner(pane, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setDigits(1);
		sp.setMinimum(0);
		sp.setMaximum(40);
		sp.setIncrement(5);
		sp.setSelection(0);
		sp.setData(Boolean.FALSE);
		sp.addListener(SWT.Modify, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("Composite:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Combo composites = new Combo(pane, SWT.READ_ONLY);
		composites.setItems(CompositingPart.labels);
		composites.select(3);
		composites.setData(Boolean.TRUE);
		composites.setVisibleItemCount(CompositingPart.labels.length / 2);
		composites.addListener(SWT.Selection, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("Alpha:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Spinner alphaFld = new Spinner(pane, SWT.BORDER);
		alphaFld.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		alphaFld.setMinimum(0);
		alphaFld.setMaximum(255);
		alphaFld.setIncrement(1);
		alphaFld.setSelection(255);
		alphaFld.setData(new Integer(0));
		alphaFld.addListener(SWT.Modify, listener);
		
		return container;
	}
	
	protected void openImage() {
		FileDialog dialog = new FileDialog(client.getShell(), SWT.OPEN);
		dialog.setFilterNames(IMG_NAMES);
		dialog.setFilterExtensions(IMG_EXTS);
		String name = dialog.open();
		if (name != null) {
			InputStream in = null;
			try {
				in = new FileInputStream(name);
				image = getImage(in);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} finally {
				if (in != null) {
					try {
						in.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			redraw();
		}
	}
}
