package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.Area;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import com.spket.g2d.G2DRenderingHints;
import com.spket.g2d.Gradient;
import com.spket.g2d.LinePattern;
import com.spket.g2d.NativeImage;
import com.spket.widget.ColorPicker;

public class LinePatternPart extends Part {
	private static final float[] DASH = { 20, 6 };
	private static final int[] RATIOS = { 0, 85, 170, 255 };
	private static final Color[] COLORS = { Color.red, Color.yellow, Color.green, Color.blue };
	
	private class LinePatternRenderer extends Renderer {
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			int w1 = w / 4;
			int h1 = h / 4;
			
			if (applyClip) {
				GeneralPath path = new GeneralPath(new Ellipse2D.Float(20, 20, w / 2, h - 40));
				path.append(new Ellipse2D.Float(w1 - 10, h1 - 10, w / 2 + 20, h / 2 + 20), false);
				path.append(new Rectangle2D.Float((w + w1) / 2, h1 / 2, w1, h - h1), false);
				
				Area area = new Area(path);
				area.subtract(new Area(new Ellipse2D.Float((w - w1) / 2, (h - h1) / 2, w1, h1)));
				
				g2.setClip(area);
				
				//g2.setClip(20, 20, w - 30, h - 30);
				g2.setRenderingHint(G2DRenderingHints.KEY_CLIP_ANTIALIASING, G2DRenderingHints.VALUE_CLIP_ANTIALIASING_ON);
			}
			
			if (background != null) {
				Color c = g2.getBackground();
				g2.setBackground(background);
				g2.clearRect(0, 0, w, h);
				g2.setBackground(c);
			}
			
			g2.setRenderingHint(G2DRenderingHints.KEY_IMAGE_FILTER, G2DRenderingHints.VALUE_FILTER_BILINEAR);
			
			if (applyComposite)
				g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
			
			g2.setPaint(new LinePattern(p0));
			g2.drawArc(20, 20, w - 40, h - 40, 0, 180);
			
			g2.setPaint(new LinePattern(p1));
			g2.drawArc(w1, h1, w / 2, h / 2, 0, 270);
			
			Stroke s = g2.getStroke();
			g2.setStroke(new BasicStroke(10, BasicStroke.CAP_BUTT, BasicStroke.JOIN_ROUND, 0, DASH, 0));
			g2.setPaint(Color.yellow);
			g2.draw(new CubicCurve2D.Float(10, h / 2, w1, 0, w - w1, h, w - 10, h / 2));
			
			g2.setPaint(new LinePattern(p2));
			g2.draw(new CubicCurve2D.Float(10, h / 2, w1, 0, w - w1, h, w - 10, h / 2));
			g2.setStroke(s);
			
			g2.setPaint(new LinePattern(p3));
			g2.draw(new CubicCurve2D.Float(w / 2, 10, w1, h1, w - w1, h - h1, w / 2, h - 10));
			
			g2.setPaint(new LinePattern(p4));
			g2.draw(new CubicCurve2D.Float(10, h - 10, w1, h, w - w1, 0, w - 10, 10));
			
			/*
			if (texture != null) {
				g2.setPaint(new LinePattern(texture));
			
				g2.drawOval(10, 10, w - 20, h - 20);
			}
			*/
		}

		public boolean support(boolean awt) {
			return !awt;
		}
	}
	
	private Control client;
	
	private NativeImage  p0;
	private BufferedImage p1;
	private BufferedImage p2;
	private BufferedImage p3;
	private BufferedImage p4;
	//private BufferedImage texture;

	private Color background;
	private boolean applyClip;
	private boolean applyComposite;

	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			Object d = event.widget.getData();
			switch (event.type) {
				case SWT.Selection:
					if (Boolean.FALSE == d) {
						applyClip = !applyClip;
						redraw();
					} else if (Boolean.TRUE == d) {
						applyComposite = !applyComposite;
						redraw();
					}
					break;
				case SWT.Modify:
					if (event.detail == -1)
						background = null;
					else
						background = new Color(event.detail);
					redraw();
					break;
			}
		}
	};
	
	public void dispose() {
		if (p0 != null) {
			p0.dispose();
			p0 = null;
		}
		super.dispose();
	}

	protected Control getClient() {
		return client;
	}

	protected Renderer createRenderer() {
		p0 = new NativeImage(10, 20, true);
		Graphics2D g = p0.createGraphics();
		g.setPaint(new Gradient(0, 0, 0, p0.getHeight(), RATIOS, COLORS));
		g.fillRect(0, 0, p0.getWidth(), p0.getHeight());
		g.dispose();
		
		p1 = getImage("/com/spket/test/images/p1.gif");
		p2 = getImage("/com/spket/test/images/p2.gif");
		p3 = getImage("/com/spket/test/images/p3.gif");
		p4 = getImage("/com/spket/test/images/p4.gif");
		
		return new LinePatternRenderer();
	}

	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout(2, false));
		
		client = new Label(container, SWT.NONE);
		client.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout(2, false));
		pane.setLayoutData(new GridData(GridData.FILL_VERTICAL));
		/*
		Label label = new Label(pane, SWT.NONE);
		label.setText("Image:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Button button = new Button(pane, SWT.PUSH);
		button.setText("Browser...");
		button.setData(Boolean.FALSE);
		button.addListener(SWT.Selection, listener);
		*/
		Label label = new Label(pane, SWT.NONE);
		label.setText("Background:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		ColorPicker picker = new ColorPicker(pane, SWT.FLAT);
		picker.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_CENTER | GridData.VERTICAL_ALIGN_CENTER));
		picker.addListener(SWT.Modify, listener);
		
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		
		Button button = new Button(pane, SWT.CHECK);
		button.setLayoutData(gd);
		button.setText("Clip");
		button.setData(Boolean.FALSE);
		button.setSelection(applyClip);
		button.addListener(SWT.Selection, listener);

		button = new Button(pane, SWT.CHECK);
		gd = new GridData();
		gd.horizontalSpan = 2;		button.setLayoutData(gd);
		button.setText("Composite");
		button.setData(Boolean.TRUE);
		button.setSelection(applyClip);
		button.addListener(SWT.Selection, listener);
		
		return container;
	}
	/*
	protected void openImage() {
		FileDialog dialog = new FileDialog(client.getShell(), SWT.OPEN);
		dialog.setFilterNames(IMG_NAMES);
		dialog.setFilterExtensions(IMG_EXTS);
		String name = dialog.open();
		if (name != null) {
			InputStream in = null;
			try {
				in = new FileInputStream(name);
				texture = getImage(in);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} finally {
				if (in != null) {
					try {
						in.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
			redraw();
		}
	}
	*/
}
