package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Widget;

import com.spket.g2d.SWTImage;

public abstract class Part implements IPart {
	protected static final byte[] gBuffer = new byte[1024];
	
	protected static final String TEXTURE_URL = "/com/spket/test/images/updates_obj.gif";
	
	protected static final String[] IMG_NAMES = { "All supported files", "All files(*.*)" };
	protected static final String[] IMG_EXTS = { "*.png;*.gif;*.jpg;*.bmp;*.ico;*.tif", "*.*" };
	
	protected static BufferedImage gTexture;
	protected static BufferedImage gBackground;
	
	public static BufferedImage getImage(String url) {
		BufferedImage image = null;
		InputStream in = Part.class.getResourceAsStream(url);
		if (in != null) {
			try {
				image = getImage(in);
			} finally {
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		
		return image;
	}
	
	public static BufferedImage getImage(InputStream in) {
		try {
			return new SWTImage(in).convert();
		} catch (Throwable t) {
			t.printStackTrace();
		}
		return null;
	}
	
	protected static boolean okToUse(Widget widget) {
		return (widget != null && !widget.isDisposed());
	}
	
	private static void drawLine(Graphics2D g, int w, int h, int start, int space) {
		g.setPaint(Color.red);
		for (int y = start; y < 340; y += 50)
			g.drawLine(0, y, 340, y);
		
		start += space;
		g.setPaint(Color.green);
		for (int y = start; y < 340; y += 50)
			g.drawLine(0, y, 340, y);
		
		start += space;
		g.setPaint(Color.blue);
		for (int y = start; y < 340; y += 50)
			g.drawLine(0, y, 340, y);
	}
	
	private Control control;
	private Renderer renderer;
	
	public final Control createControl(Composite parent) {
		control = createPane(parent); 
		return control;
	}

	public void dispose() {
		if (control != null) {
			control.dispose();
			control = null;
		}
		renderer = null;
	}

	public Rectangle getClientArea() {
		Control client = getClient();
		if (client != null) {
			Rectangle rect;
			if (client instanceof Composite) {
				rect = ((Composite) client).getClientArea();
			} else {
				rect = client.getBounds();
				client = client.getParent();
			}
			if (client != control) {
				Point p = getControl().toControl(client.toDisplay(rect.x, rect.y));
				rect.x = p.x;
				rect.y = p.y;
			}
			return rect;
		}

		return null;
	}

	public final Control getControl() {
		return control;
	}

	public final Renderer getRenderer() {
		if (renderer == null)
			renderer = createRenderer();
		
		return renderer;
	}

	public void setFocus() {
		if (control != null)
			control.setFocus();
	}
	
	public void addResizeListener(Listener listener) {
		Control control = getClient();
		if (okToUse(control))
			control.addListener(SWT.Resize, listener);
	}
	
	public void removeResizeListener(Listener listener) {
		Control control = getClient();
		if (okToUse(control))
			control.removeListener(SWT.Resize, listener);
	}
	
	protected BufferedImage getTexture() {
		if (gTexture == null)
			gTexture = getImage(TEXTURE_URL);
		
		return gTexture;
	}
	
	protected BufferedImage getDefaultImage() {
		//if (gBackground == null) {
			gBackground = new BufferedImage(340, 340, BufferedImage.TYPE_INT_ARGB);
			CompositingPart part = new CompositingPart();
			part.composite = AlphaComposite.DST_ATOP;
			Renderer ren = part.getRenderer();
			Graphics2D g = gBackground.createGraphics();
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
			
			drawLine(g,gBackground.getWidth(), gBackground.getHeight(), 2, 1);
			drawLine(g,gBackground.getWidth(), gBackground.getHeight(), 7, 2);
			g.setStroke(new BasicStroke(2));
			drawLine(g,gBackground.getWidth(), gBackground.getHeight(), 16, 2);
			drawLine(g,gBackground.getWidth(), gBackground.getHeight(), 26, 3);
			drawLine(g,gBackground.getWidth(), gBackground.getHeight(), 39, 4);
			ren.render(g, true, gBackground.getWidth(), gBackground.getHeight());
			g.dispose();
		//}
		
		return gBackground;
	}

	protected void redraw() {
		if (renderer != null)
			renderer.redraw();
	}
	
	protected Control getClient() {
		return getControl();
	}
	
	protected GridLayout createLayout() {
		return createLayout(1, false);
	}
	
	protected GridLayout createLayout(int numColumns, boolean makeColumnsEqualWidth) {
		GridLayout layout = new GridLayout(numColumns, makeColumnsEqualWidth);
		layout.marginWidth = layout.marginHeight = 0;
		layout.horizontalSpacing = layout.verticalSpacing = 2;
		return layout;
	}
	
	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		return container;
	}
	
	protected abstract Renderer createRenderer();
}
