package com.spket.test;

import java.awt.AlphaComposite;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

import org.apache.batik.ext.awt.image.SVGComposite;

import com.spket.g2d.G2DComposite;
import com.spket.g2d.internal.G2D;

public abstract class Renderer {
	public static void draw(Graphics2D g2, Shape shape) {
		AffineTransform mtx = g2.getTransform();
		g2.setTransform(new AffineTransform());
		g2.draw(mtx.createTransformedShape(shape));
		g2.setTransform(mtx);
	}
	
	protected static void drawString(Graphics2D g2, String text, double width, double height) {
		if (text == null)
			return;
		
		TextLayout tl = new TextLayout(text, g2.getFont(), g2.getFontRenderContext());
		Rectangle2D bounds = tl.getBounds();
		//g2.translate(
		tl.draw(g2, (float) (width - bounds.getWidth()) / 2, (float) (height - bounds.getHeight()) / 2);
	}
	
	protected static Composite getComposite(boolean awt, int rule, int alpha) {
		float a = alpha / 255.0f;
		if (awt) {
			if (rule <= AlphaComposite.XOR) {
				return AlphaComposite.getInstance(rule, a);
			} else {
				switch (rule) {
					case G2DComposite.MULTIPLY:
						return SVGComposite.MULTIPLY;
					case G2DComposite.SCREEN:
						return SVGComposite.SCREEN;
					case G2DComposite.DARKEN:
						return SVGComposite.DARKEN;
					case G2DComposite.LIGHTEN:
						return SVGComposite.LIGHTEN;
				}
			}
			return null;
		}
		return G2DComposite.getInstance(rule, a);
	}
	
	protected static Object getFilter(boolean awt, int filter) {
		if (awt) {
			switch (filter) {
				case 1:
					return RenderingHints.VALUE_INTERPOLATION_BILINEAR;
				case 2:
					return RenderingHints.VALUE_INTERPOLATION_BICUBIC;
			}
			return RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR;
		}
		
		return new Integer(filter);
	}
	
	protected static boolean supportComposite(boolean awt, int rule, int alpha) {
		if (awt) {
			if (rule <= AlphaComposite.XOR)
				return true;
			
			if (alpha == 255) {
				switch (rule) {
					case G2DComposite.MULTIPLY:
					case G2DComposite.SCREEN:
					case G2DComposite.DARKEN:
					case G2DComposite.LIGHTEN:
						return true;
				}
			}
			return false;
		}
		return true;
	}
	
	protected static boolean supportFilter(boolean awt, int filter) {
		if (awt)
			return filter <= G2D.BICUBIC;
		
		return true;
	}
	
	private G2Demo host;
	
	public void redraw() {
		if (host != null)
			host.redraw();
	}
	
	public void setHost(G2Demo host) {
		this.host = host;
	}
	
	public boolean support(boolean awt) {
		return true;
	}
	
	public abstract void render(Graphics2D g2, boolean awt, int w, int h);
}
