package com.spket.test;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.TexturePaint;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;

import com.spket.g2d.Texture;
import com.spket.g2d.internal.G2D;

public class TexturePart extends Part {
	private final class TextureRenderer extends Renderer {
		public void render(Graphics2D g2, boolean awt, int w, int h) {
			BufferedImage image = getTexture();
			if (image == null)
				return;
			
			if (rotation != 0)
				g2.rotate(Math.toRadians(rotation), w / 2, h / 2);
			
			g2.setPaint(Color.lightGray);
			g2.fillOval(0, 0, w, h);
			if (!awt && texture == null) {
				g2.setPaint(new Texture(image, new Rectangle2D.Double(0, 0, image.getWidth() * scale, image.getHeight() * scale), xReflect, yReflect));
			} else {
				if (texture == null) {
					int width = image.getWidth();
					int height = image.getHeight();
					
					int w1 = width, h1 = height;
					if (xReflect == Texture.REFLECT)
						w1 *= 2;
					if (yReflect == Texture.REFLECT)
						h1 *= 2;
					
					if (xReflect == Texture.REPEAT && yReflect == Texture.REPEAT) {
						texture = image;
					} else {
						texture = new BufferedImage(w1, h1, BufferedImage.TYPE_INT_ARGB);
						Graphics2D g = texture.createGraphics();
						g.setPaint(new Color(0, 0, 0, 0));
						g.fillRect(0, 0, w1, h1);
						g.drawImage(image, 0, 0, null);
						if (xReflect == Texture.REFLECT) {
							g.translate(w1, 0);
							g.scale(-1, 1);
							g.drawImage(image, 0, 0, null);
							if (yReflect == Texture.REFLECT) {
								g.setTransform(new AffineTransform());
								g.translate(0, h1);
								g.scale(1, -1);
								g.drawImage(image, 0, 0, null);
								g.setTransform(new AffineTransform());
								g.translate(w1, h1);
								g.scale(-1, -1);
								g.drawImage(image, 0, 0, null);
							}
						} else if (yReflect == Texture.REFLECT) {
							g.translate(0, h1);
							g.scale(1, -1);
							g.drawImage(image, 0, 0, null);
						}
											
						g.dispose();
					}
				}
				g2.setPaint(new TexturePaint(texture, new Rectangle2D.Double(0, 0, texture.getWidth() * scale, texture.getHeight() * scale)));
			}
			g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, getFilter(true, filter));
			g2.fillOval(0, 0, w, h);
		}
	}
	
	private Label client;
	
	private int rotation;
	private double scale = 1.0;
	private byte xReflect = Texture.REPEAT;
	private byte yReflect = Texture.REPEAT;
	
	private BufferedImage texture;
	private int filter = G2D.NEAREST_NEIGHBOR;
	
	private Listener listener = new Listener() {
		public void handleEvent(Event event) {
			if (SWT.Selection == event.type) {
				if (event.widget instanceof Button) {
					texture = null;
					onButtonClick(event);
				} else {
					int v = ((Combo) event.widget).getSelectionIndex();
					if (v != filter) {
						filter = v;
						redraw();
					}
				}
			} else {
				Spinner sp = (Spinner) event.widget;
				int v = sp.getSelection();
				if (sp.getData() == Boolean.TRUE) {
					double s = v / 10.0;
					if (s != scale) {
						scale = s;
						redraw();
					}
				} else {
					if (v != rotation) {
						rotation = v;
						texture = null;
						redraw();
					}
				}
			}
		}
	};
	
	protected Renderer createRenderer() {
		return new TextureRenderer();
	}

	protected Control getClient() {
		return client;
	}

	protected Control createPane(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(createLayout());
		
		Composite pane = new Composite(container, SWT.NONE);
		pane.setLayout(new GridLayout(8, false));
		pane.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		
		Button button = new Button(pane, SWT.CHECK);
		button.setText("reflect(&X)");
		button.setSelection(xReflect == Texture.REFLECT);
		button.setData(Boolean.TRUE);
		button.addListener(SWT.Selection, listener);
		
		button = new Button(pane, SWT.CHECK);
		button.setText("reflect(&Y)");
		button.setSelection(xReflect == Texture.REFLECT);
		button.setData(Boolean.FALSE);
		button.addListener(SWT.Selection, listener);
		
		Label label = new Label(pane, SWT.NONE);
		label.setText("&Rotation:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Spinner sp = new Spinner(pane, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setMinimum(0);
		sp.setMaximum(360);
		sp.setIncrement(10);
		sp.setSelection(0);
		sp.addListener(SWT.Modify, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("&Scale:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		sp = new Spinner(pane, SWT.BORDER);
		sp.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		sp.setDigits(1);
		sp.setMinimum(5);
		sp.setMaximum(40);
		sp.setIncrement(5);
		sp.setSelection(10);
		sp.setData(Boolean.TRUE);
		sp.addListener(SWT.Modify, listener);
		
		label = new Label(pane, SWT.NONE);
		label.setText("F&ilter:");
		label.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_END));
		
		Combo combo = new Combo(pane, SWT.READ_ONLY | SWT.DROP_DOWN);
		combo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		combo.add("nearest neighbor");
		combo.add("bilinear");
		combo.add("bicubic");
		combo.select(0);
		combo.addListener(SWT.Selection, listener);

		client = new Label(container, SWT.NONE);
		client.setLayoutData(new GridData(GridData.FILL_BOTH));

		return container;
	}
	
	protected void onButtonClick(Event event) {
		if (event.widget.getData() == Boolean.TRUE) {
			xReflect = (xReflect == Texture.REFLECT) ? Texture.REPEAT : Texture.REFLECT;
			redraw();
		} else {
			yReflect = (yReflect == Texture.REFLECT) ? Texture.REPEAT : Texture.REFLECT;
			redraw();
		}
	}
}
